######################################################
#' @title Figure 4 -  Within group DEP prevalance of top five ranked pathways
#' @author A. Grant Schissler
#' @date 8 Dec 2015
#' @description XXX
######################################################

######################################################
setwd("./Figure4")
rm(list=ls())
load("../CommonData/CTC_pairs.RData")

### lab colors:
purple <- "#A078AA"
orange <- "#FFA040"
green <- "#B72BF44"
grey <- "#777676"
yellow <- "#F9F080"

#### retrieve percent dysregulated
## create binary indicator for pathway
## tmp.data <- data.list[[5]]
threshold <- 0.05
sig.indicator.list <- lapply(data.list, function(tmp.data) {
    tmp.data <- tmp.data[pid.id,]
    as.numeric(tmp.data$BY < threshold & !is.nan(tmp.data$distance))
})

sig.indicator.matrix <- do.call("rbind",sig.indicator.list)
colnames(sig.indicator.matrix) <- pid.id

#### restrict to the top five pathways
## top pathway IDs
top.ids <- c("200020", "200141", "200135", "200142", "200085")
top.labels <- c("ncWnt","ErbB2/B3","SDC4", "FOXM1", "S1P1")
top.ind <- sig.indicator.matrix[,top.ids]
## uncomment this to summarize all pathways
## top.ind <- sig.indicator.matrix

### create data frame for plotting 
top.ind.data <- data.frame(top.ind)

## label remaining pairings
top.ind.data$Pairing <- "EMPTY"
top.ind.data[a.a.index,"Pairing"] <- "N"
top.ind.data[b.b.index,"Pairing"] <- "R"
top.ind.data[b.a.index,"Pairing"] <- "B-A"

## remove B-A comparisons
top.ind.data <- top.ind.data[-b.a.index,]

## remove within patient comparisons
patient.pairs <- unlist(lapply(strsplit(rownames(top.ind.data), "[\\.|-]"), function(tmp) tmp[1] == tmp[4]))
top.ind.data <- top.ind.data[!patient.pairs,]

### summarize to write in long format to construct bar plot
## technically a weighted cross-patinent observed proportion
top.ind.long <- data.frame(do.call("rbind",lapply(split(top.ind.data, top.ind.data$Pairing), function(df) {
    apply(df[,-ncol(df)], 2, function(tmp.geneset) {
        100*(sum(tmp.geneset)/length(tmp.geneset))
    })
})))
top.ind.long$Pairing <- row.names(top.ind.long)

## melt
require(reshape2)
names(top.ind.long) <- gsub("^X","",names(top.ind.long))
top.ind.long <- melt(top.ind.long)
names(top.ind.long) <- c("Pairing","PID","PercentDysregulated")
top.ind.long$PID <- factor(top.ind.long$PID, labels = top.labels)

## sumarize all pathways
## all.long <- top.ind.long
## sum(all.long[top.ind.long$Pairing == "R",3] <= mean(top.ind.long[top.ind.long$Pairing == "R",3]))/nrow(all.long[top.ind.long$Pairing == "R",])

## sum(all.long[top.ind.long$Pairing == "N",3] <= mean(top.ind.long[top.ind.long$Pairing == "N",3]))/nrow(all.long[top.ind.long$Pairing == "N",])

####################
#### create error bars by clustered bootstrap resampling
bootrep <- 5000
n1 <- length(a.patients)
n2 <- length(b.patients)
top.ind.matrix <- as.matrix(top.ind.data[,-ncol(top.ind.data)])
colnames(top.ind.matrix) <- top.labels

## create variables to populate later
top.ind.long$lower <- NA
top.ind.long$upper <- NA

## set seed
set.seed(440)

start.time <- proc.time()
for (ii in colnames(top.ind.matrix)) {
    ## create vectors for storage
    boot.prop.a <- NULL
    boot.prop.b <- NULL
    ## index pathway
    tmp.geneset <- top.ind.matrix[,ii]
    ## bootstrap for this pathway
    for (i in 1:bootrep) {
        ## complete A patients first
        tmp.a <- sample(a.patients, n1, replace = T)
        tmp.prop.a <- NULL
        for (aa in tmp.a) {
            tmp.prop.a <- c(tmp.prop.a,tmp.geneset[grep(aa, names(tmp.geneset))])
        }
        boot.prop.a <- c(boot.prop.a,100*sum(tmp.prop.a)/length(tmp.prop.a))
        ## then complete B patients
        tmp.b <- sample(b.patients, n2, replace = T)
        tmp.prop.b <- NULL
        for (bb in tmp.b) {
            tmp.prop.b <- c(tmp.prop.b,100*tmp.geneset[grep(bb, names(tmp.geneset))])
        }
        boot.prop.b <- c(boot.prop.b,sum(tmp.prop.b)/length(tmp.prop.b))
    }
    ## summarize by finding confidence limits
    ci.a <- quantile(boot.prop.a,c(0.025,0.975))
    ci.b <- quantile(boot.prop.b,c(0.025,0.975))
    ## store the bootstrapped percentile 95% confidence interval
    top.ind.long[top.ind.long$PID == ii & top.ind.long$Pairing == "N" , "lower"] <- ci.a[1]
    top.ind.long[top.ind.long$PID == ii & top.ind.long$Pairing == "N" , "upper"] <- ci.a[2]
    top.ind.long[top.ind.long$PID == ii & top.ind.long$Pairing == "R" , "lower"] <- ci.b[1]
    top.ind.long[top.ind.long$PID == ii & top.ind.long$Pairing == "R" , "upper"] <- ci.b[2]
}
(end.time <- proc.time() - start.time)

## took 3 minutes for 5000 bootstraps

## save workspace so I don't have to recompute
save.image("Figure_Dysreg_Group_bootstrap.RData")

#################################################################################
######### figure version 3
load("./Figure_Dysreg_Group_bootstrap.RData")

### reverse the order of pathways
top.ind.long$PID <- factor(top.ind.long$PID, levels = levels(top.ind.long$PID)[5:1])

#### create plot with points instead of bars
p0 <- ggplot(top.ind.long, aes(x=PID, y= PercentDysregulated, shape=Pairing, col = Pairing))

### add points with error bars
(p1 <- p0 + geom_point(size = 3, stat = "identity") + theme_bw() + theme(legend.position = "bottom"))

### add error bars
(p2 <- p1 + geom_errorbar(aes(ymax = upper, ymin = lower), width=0.1))

### annotate
(p3 <- p2 + scale_shape("Paring Within Group:",labels = c("Naive","Resistant")) +
 xlab("Resistance-Associated Pathways") + ylab("Percent of Dysregulated CTCs"))

## add colors
(p4 <- p3 + scale_color_manual("Paring Within Group:",labels = c("Naive","Resistant"),
                       values = c(orange,purple)))
### flip axises
(p5 <- p4 + coord_flip())

## use ggsave to avoid problems with shapes
ggsave(plot=p5 ,height=6, width=6, dpi=200, filename="Figure4.pdf", useDingbats=FALSE)
